@@ -10,7 +10,7 @@
 
 #include "all_types.hpp"
 
-IrInstGen *ir_create_alloca(CodeGen *g, Scope *scope, AstNode *source_node, ZigFn *fn,
+Stage1AirInst *ir_create_alloca(CodeGen *g, Scope *scope, AstNode *source_node, ZigFn *fn,
         ZigType *var_type, const char *name_hint);
 
 Error ir_eval_const_value(CodeGen *codegen, Scope *scope, AstNode *node,
@@ -25,7 +25,7 @@ ZigType *ir_analyze(CodeGen *codegen, Stage1Zir *stage1_zir, Stage1Air *stage1_a
         ZigType *expected_type, AstNode *expected_type_source_node, ZigValue *result_ptr,
         ZigFn *fn);
 
-bool ir_inst_gen_has_side_effects(IrInstGen *inst);
+bool ir_inst_gen_has_side_effects(Stage1AirInst *inst);
 
 struct IrAnalyze;
 ZigValue *const_ptr_pointee(IrAnalyze *ira, CodeGen *codegen, ZigValue *const_val,
