@@ -21,6 +21,20 @@ static inline float16_t zig_double_to_f16(double x) {
     return f64_to_f16(y);
 }
 
+static inline void zig_double_to_extF80M(double x, extFloat80_t *result) {
+    float64_t y;
+    static_assert(sizeof(x) == sizeof(y), "");
+    memcpy(&y, &x, sizeof(x));
+    f64_to_extF80M(y, result);
+}
+
+static inline void zig_double_to_f128M(double x, float128_t *result) {
+    float64_t y;
+    static_assert(sizeof(x) == sizeof(y), "");
+    memcpy(&y, &x, sizeof(x));
+    f64_to_f128M(y, result);
+}
+
 
 // Return value is safe to coerce to float even when |x| is NaN or Infinity.
 static inline double zig_f16_to_double(float16_t x) {
@@ -56,4 +70,8 @@ static inline bool zig_f128_isNaN(float128_t *aPtr) {
             || ((absA64 == UINT64_C(0x7FFF000000000000)) && lo);
 }
 
+static inline bool zig_extF80_isNaN(extFloat80_t *aPtr) {
+    return (aPtr->signExp & 0x7FFF) == 0x7FFF && aPtr->signif & UINT64_C(0x7FFFFFFFFFFFFFFF);
+}
+
 #endif
